@@ -35,8 +35,11 @@ THE SOFTWARE.
 import numpy as np
 from scipy.interpolate import interp1d
 from scipy.optimize import minimize
+import scipy.integrate as sp 
+from scipy.interpolate import InterpolatedUnivariateSpline as spline
 import warnings as _warnings
 
+
 # Define global constants
 from . import const
 PC_SI     = const.PCinm   # Parsec in meters
@@ -959,26 +962,34 @@ def compute_many_mode_errors(h,h_ref,mode_list,mynorm):
   return relative_mode_errs
 
 
-def interpolant_h(t, h, deg=3):
-  """Compute spline interpolant of input data h at samples t."""
+def interpolant_h(t, h, deg=3, s=None):
+  """Compute spline interpolant of input data h at samples t.
+
+  s is the spline smoothing parameter
+
+  """
   
   from scipy.interpolate import splrep
   from scipy.interpolate import splev
   dtype = h.dtype
   if dtype == 'complex':
-    interp_real = splrep(t, np.real(h), k=deg)
-    interp_imag = splrep(t, np.imag(h), k=deg)
+    interp_real = splrep(t, np.real(h), k=deg, s=s)
+    interp_imag = splrep(t, np.imag(h), k=deg, s=s)
     return interp_real, interp_imag
   elif dtype == 'double':
-    interp = splrep(t, h, k=deg)
+    interp = splrep(t, h, k=deg, s=s)
     return interp
   else:
     raise Exception("Function to be interpolated must be real or complex.")
   pass
 
 
-def interpolate_h(tin, h, tout, deg=3):
-  """Interpolate input data h at samples tout."""
+def interpolate_h(tin, h, tout, deg=3, s=None):
+  """Interpolate input data h at samples tout.
+
+  s is the spline smooth parameter
+
+  """
 
   from scipy.interpolate import splrep
   from scipy.interpolate import splev
@@ -988,17 +999,17 @@ def interpolate_h(tin, h, tout, deg=3):
   
   dtype = h.dtype
   if dtype == 'complex':
-    interp_real, interp_imag = interpolant_h(tin, h, deg=deg)
+    interp_real, interp_imag = interpolant_h(tin, h, deg=deg,s=s)
     return splev(tout, interp_real) + 1j*splev(tout, interp_imag)
   elif dtype == 'double':
-    interp = interpolant_h(tin, h, deg=deg)
+    interp = interpolant_h(tin, h, deg=deg,s=s)
     return splev(tout, interp)
   else:
     raise Exception("Function to be interpolated must be real or complex.")
   pass
 
 
-def spline_f(tin, h, deg=3):
+def spline_f(tin, h, deg=3, s=None):
   """ Return a function f_sp such that f_sp(t) returns the spline
       evaluation of f"""
 
@@ -1007,9 +1018,9 @@ def spline_f(tin, h, deg=3):
 
   dtype = h.dtype
   if dtype == 'complex':
-    interp_real, interp_imag = interpolant_h(tin, h, deg=deg)
+    interp_real, interp_imag = interpolant_h(tin, h, deg=deg, s=s)
   elif dtype == 'double':
-    interp = interpolant_h(tin, h, deg=deg)
+    interp = interpolant_h(tin, h, deg=deg, s=s)
   else:
     raise Exception("Function to be interpolated must be real or complex.")
 
@@ -1029,6 +1040,142 @@ def spline_f(tin, h, deg=3):
   return f_sp
 
 
+def energy_flux_from_modes(h, time, q, M=1., tend=100.):
+  """ Computes the radiated energy and flux from each mode (E_modes), and 
+    total energy (E_tot) as described in Eq.(2) of arXiv:1802.04276v2 and E0_over_M (Eq. 3). 
+        
+    We assume equal contributions from m>0 and m<0 modes, 
+    which is only true for binary systems that orbit in the x-y plane.
+        
+    INPUT
+    =====
+    h:  The complex gravitational strain as a dictionary of modes.
+        Example: h[(2,2)] is an numpy array
+    time: A numpy array, times at which the gravitational strain is calculated.
+    M:    Total mass of the binary at the begining of the evolution. Default: 1.0 
+    tend: compute up to this final time (in units of M)
+
+    RETURNS
+    =====
+    E: A dictionary which contains...
+      (1) E['total'], the total radiated energy by graviational waves over different modes. This 
+          Does not include a PN estimated value E0_over_M for flux at E(0)
+      (2) E[(l, m)] the total energy radiated that does include a PN estimated value E0_over_M for flux at E(0)
+
+
+    EXAMPLE
+    =======
+    # load model
+    import gwsurrogate as gws
+    model = gws.LoadSurrogate("NRHybSur3dq8")
+
+    # import routine
+    from gwtools.gwtools import energy_flux_from_modes
+
+    # evaluate model
+    q=2.0
+    times, h, dynamics = model(q=q,chiA0=[0,0,0],chiB0=[0,0,0], dt=0.1, units='dimensionless', f_low=.005)
+
+    # Compute flux
+    E_fluxes = energy_flux_from_modes(h=h, time=times, q=q, M=1., tend=100)
+
+    # plot flux
+    import matplotlib.pyplot as plt
+    plt.plot(times, E_fluxes['total'],'black',label='total energy flux')
+    plt.plot(times, E_fluxes[(2,2)],'blue',label='energy flux from 22 mode only')
+    plt.plot(times, E_fluxes[(2,-2)],'r--',label='energy flux from 22 mode only')
+    plt.legend()
+           
+           
+    NOTE
+    ====
+    After this code was written, it was realized that similar code is alredy online: 
+
+           https://github.com/dgerosa/surrkick
+
+    Future work: directly compare this code to the one above (see https://github.com/dgerosa/surrkick/blob/master/surrkick/surrkick.py#L283)
+
+    Author: Estuti Shukla
+    Date: 9/10/2021
+  """
+    
+  # Final time in units of M
+  tend=tend*M
+
+  #finite differences
+  hdot={j:np.gradient(h[j],edge_order=2)/np.gradient(time,edge_order=2) for j in h.keys()}
+
+  #from surkick model code https://github.com/dgerosa/surrkick/blob/master/surrkick/surrkick.py
+  #hdot=  {k: spline(time,v.real).derivative()(time)+1j*spline(time,v.imag).derivative()(time) for k, v in h.items()}
+
+  dEdt={'total':0}
+
+  for i in h.keys():
+    (l,m)=i
+    #ensures that m >=0 to avoid any error in calculation. m<0 are redundant in this case of orbital symmetry and already accouted for in the calcualtions below.
+    assert(m>=0),"Negative value of m" 
+    dEdt[i]=(1/(16*np.pi))*(np.abs(hdot[i])**2) # dEdt using the equation 2 of  arXiv:1802.04276v2
+    if m!=0:
+      # multiplied by 2 to account for both positive and negative modes  
+      dEdt['total']+=dEdt[i]*2
+      dEdt[(l,-m)] = dEdt[i]
+    else:
+      dEdt['total']+=dEdt[i]
+
+  E={}
+  for j in dEdt.keys():
+    #from surkick model code https://github.com/dgerosa/surrkick/blob/master/surrkick/surrkick.py
+    #E[j]=spline(time,dEdt[j]).antiderivative()(time)
+
+    #E[j]=sp.cumtrapz(dEdt[j],time)
+    E[j]=sp.cumtrapz(dEdt[j],time,initial=0.0)
+    #print(E[j].shape)
+    #print(dEdt[j].shape)
+    #print(time.shape)
+
+  tf=time[0]+tend
+  time1=time[time<tf]
+  #from surkick model code https://github.com/dgerosa/surrkick/blob/master/surrkick/surrkick.py
+  #Edot0=spline(time[time<tf],dEdt['total'][time<tf]).antiderivative()(time1[-1])/((time1[-1]-time1[0])*M)
+
+
+  Edot0=sp.trapz(dEdt['total'][0:len(time1)],time1)/((time1[-1]-time1[0])*M)
+  E0_over_M=((5./1024.)*((q**3.)/(1.+q)**6.)*Edot0)**(1./5.)  #A float type number as described by eq 3. in arXiv:1802.04276v2.
+
+  E['total']+=E0_over_M*M
+
+  return E
+
+
+
+def bondi_mass_binary(E, M=1.):
+  """ Computes Bondi mass as described in Eq.(4) of arXiv:1802.04276v2. Returns M_t as an array. 
+
+  INPUT
+  =====
+  E: An dictionary which contains the energy radiated over
+  all the modes (as well as the total energy) by the gravitational wave.
+  M: Total mass of the binary at the begining of the evolution. Default: 1.0 
+
+  RETURNS
+  =====
+  M_t: An array which defines the time-dependent (Bondi) mass of the binary.
+  M_rem_over_M: A float point number which is the remnant mass or the mass of the post-merger 
+               of BH in units of the mass at early times. Shown in eq.5 of arXiv:1802.04276v2. 
+                
+  Author: Estuti Shukla
+  Date: 9/10/2021
+
+  """
+
+  M_t=M+E['total'][0]-E['total']
+
+  #final mass in units of the mass at early times (t->-infinity)
+  M_rem_over_M=1-E['total'][-1]/(1+E['total'][0])
+
+  return M_t, M_rem_over_M
+
+
 ##############################################
 # Convenient interfaces for LALsim functions #
 ##############################################
@@ -1409,7 +1556,20 @@ def modes_dict_to_list(mode_dict, mode_list=None):
 
 def modes_list_to_dict(mode_list, mode_array):
   """ Convert a numpy array indexed by list of modes into a dictionary
-  of modes."""
+  of modes. 
+
+  Assumes elements of mode_list can be used as a dictionary hash.
+
+     Ex: mode_list = [(2,2), (3,3)]
+         h[0,:]  -> 22 mode
+         h[1,:]  -> 33 mode
+
+  """
+
+  if (mode_array.shape[0] < mode_array.shape[1]):
+  	  flipped = True
+  else:
+  	  flipped = False
 
   mode_dict = {}
   for i, mode in enumerate(mode_list):
@@ -1417,6 +1577,8 @@ def modes_list_to_dict(mode_list, mode_array):
     #print(mode)
     #print(type(mode_array))
     #print(type(i))
-    mode_dict[mode] = mode_array[:,i]
-        
+    if not flipped:
+    	mode_dict[mode] = mode_array[:,i]
+    else:
+        mode_dict[mode] = mode_array[i,:]
   return mode_dict
